/*	UserInteraction.cpp    */

#include	<iostream>
#include	<fstream>
#include	<sstream>
#include	<string>

using namespace	std;

#include	"UserInteraction.h"
#include	"mystrlib.h"

void	OpenInputFile( ifstream& fin, const string& prompt, const string& default_file )
{
	string	filename;
	
	while	(true)	{
		cout	<< prompt;								// output the prompt and default choice (if any)
		if	( !default_file.empty() )
			cout << " [" << default_file << "]: ";
		else	cout << ": ";
		
		getline( cin, filename );
		
		if	( filename.empty() && !default_file.empty() )
			filename = default_file;					// substitute default if nothing typed
			
		fin.open( filename.c_str() );
		if	( !fin )	{
			fin.clear();
			cout	<< "Error opening the file." << endl;
		}
		else	break;
	}
	
	return;
}


void	OpenOutputFile( ofstream& fout, const string& prompt, const string& default_file )
{
	string	filename;
	
	while	(true)	{
		cout	<< prompt;								// output the prompt and default choice (if any)
		if	( !default_file.empty() )
			cout << " [" << default_file << "]: ";
		else	cout << ": ";
		
		getline( cin, filename );
		
		if	( filename.empty() && !default_file.empty() )
			filename = default_file;					// substitute default if nothing typed
			
		fout.open( filename.c_str() );
		if	( !fout )	{
			fout.clear();
			cout	<< "Error opening the file." << endl;
		}
		else	break;
	}
	
	return;
}


bool	AskYesOrNo( const string& prompt )
{
	string input;
	bool	response, have_response = false;
	
	do	{
		cout	<< prompt << " (yes or no)? ";
		getline( cin, input );
	
		if	( cin.fail() )	{
			// reset the input stream and try again
			cin.clear();
		}
		else	{
			// check if the response is correct
			input = strtolower( input );
			if	( input == "yes" or input == "y" )	{
				response = true;
				have_response = true;
			}
			else	if	( input == "no" or input == "n" )	{
				response = false;
				have_response = true;
			}
		}
	}
	while	( !have_response );
	
	return response;
}
	
		
bool	AskYesOrNo( const string& prompt, bool default_response )
{
	string input;
	bool	response, have_response = false;
	
	do	{
		cout	<< prompt << " (yes or no)? [" 
			<< (default_response?"yes":"no") << "] ";
		getline( cin, input );
	
		if	( cin.fail() )	{
			// reset the input stream and try again
			cin.clear();
		}
		else	{
			// check if the response is correct
			input = strtolower( input );
			if	( input == "yes" or input == "y" )	{
				response = true;
				have_response = true;
			}
			else	if	( input == "no" or input == "n" )	{
				response = false;
				have_response = true;
			}
			else	if	( input.empty() )	{
				response = default_response;
				have_response = true;
			}
		}
	}
	while	( !have_response );
	
	return response;
}


bool	QueryLine( const std::string& prompt, std::string& response, bool accept_blank )
{
	cout	<< prompt << " ";
	getline( cin, response );
	
	while	( !accept_blank && response.empty() )	{
		cout	<< prompt;
		getline( cin, response );
	}
	
	return true;
}
	
	
bool	QueryLine( const std::string& prompt, std::string& response, const std::string& default_response )
{
	cout	<< prompt << " [" << default_response << "] ";
	getline( cin, response );
	
	if	( response.empty() )	response = default_response;
		
	return true;
}
