/*	UserInteraction.h

	Anthony Kozar
	January 8, 2002
	
	A collection of helpful functions for interacting with a user in 
	the console environment.

 */	

#ifndef	ANTHONY_USER_INTERACTION_H
#define	ANTHONY_USER_INTERACTION_H

#include	<iostream>
#include	<fstream>
#include	<sstream>
#include	<string>

// These two functions provide a simple interface for asking a user for input and 
// output filenames and then opening the file.  It handles errors opening the file 
// by reprompting the user and will use default_file for the filename if the user 
// presses Enter on a blank line.
void	OpenInputFile( std::ifstream& fin, const std::string& prompt = "Enter the input filename", 
				const std::string& default_file = "" );
void	OpenOutputFile( std::ofstream& fout, const std::string& prompt = "Enter the output filename", 
				const std::string& default_file = "" );
				
// AskYesOrNo queries the user with a simple yes or no question you provide in 
// prompt and inputs his or her response as "yes", "no", "y", or "n" (ignoring case).
// The text " (yes or no)? " is added to the prompt.  You can set a default answer if 
// the user merely presses Enter (this is displayed in brackets after the prompt) in the 
// second form of the function.
bool	AskYesOrNo( const std::string& prompt );
bool	AskYesOrNo( const std::string& prompt, bool default_response );

// QueryUser inputs from cin a specific type of data after displaying the prompt
// and configurably will persist until it receives a proper response.  It returns 
// true upon success and will only return false if retry is set to false and the user
// enters an incorrect response or if some other unrecoverable error occurs.
// The default_response is returned in response if the user presses Enter on a blank
// line for the second version of the function.
template <class Type>
bool	QueryUser( const std::string& prompt, Type& response, bool retry = true );
template <class Type>
bool	QueryUser( const std::string& prompt, Type& response, const Type& default_response,
				bool retry = true );

// QueryLine inputs from cin until it encounters a newline after displaying prompt
// and returns the input line.  If the user types nothing, the first version prompts
// for input again unless accept_blank is true, while the second version simply returns
// the default_response.  Returns true when successful.
bool	QueryLine( const std::string& prompt, std::string& response, bool accept_blank = false );
bool	QueryLine( const std::string& prompt, std::string& response, const std::string& default_response );

// include the template implementations
#include	"UserInteraction_tmpl.cpp"

#endif
