/*	UserInteraction_tmpl.cpp   

	Implementations for the template functions in UserInteraction.h
	These are automatically included with the header file.
 */

template <class Type>
bool	QueryUser( const std::string& prompt, Type& response, bool retry = true )
{
	std::string input;
	std::stringstream ss;
	bool	have_response = false;
	
	do	{
		std::cout	<< prompt << " ";
		std::getline( std::cin, input );
		ss.str( input );
		ss	>> response;
	
		if	( ss.bad() || std::cin.bad() )	{
			// unrecoverable stream error
			std::cerr	<< "Error: the input stream has been corrupted." << endl;
			//exit( EXIT_FAILURE );
			return false;
		}
		else	if	( ss.fail() )	{
			// reset the input streams and try again
			std::cerr	<< "Error: your input was not formatted correctly" << (retry?", please try again.":".") << endl;
			//std::cin.clear();
			ss.str("");
			ss.clear();
		}
		else	have_response = true;
	}
	while	( !have_response && retry );
	
	return have_response;
}


template <class Type>
bool	QueryUser( const std::string& prompt, Type& response, const Type& default_response, bool retry = true )
{
	std::string input;
	std::stringstream ss;
	bool	have_response = false;
	
	do	{
		std::cout	<< prompt << " [" << default_response << "] ";
		std::getline( std::cin, input );
		if	( input.empty() )	{
			response = default_response;
			return true;
		}
		ss.str( input );
		ss	>> response;
	
		if	( ss.bad() || std::cin.bad() )	{
			// unrecoverable stream error
			std::cerr	<< "Error: the input stream has been corrupted." << endl;
			//std::exit( EXIT_FAILURE );
			return false;
		}
		else	if	( ss.fail() )	{
			// reset the input streams and try again
			std::cerr	<< "Error: your input was not formatted correctly" << (retry?", please try again.":".") << endl;
			//std::cin.clear();
			ss.str("");
			ss.clear();
		}
		else	have_response = true;
	}
	while	( !have_response && retry );
	
	return have_response;
}
