/*	AmpEnvelope.h

	An audio rate envelope which modifies the amplitude of its input.
	Capable of an arbitrary number of stages with linear transitions (no looping).
	
	Anthony Kozar
	October 8, 2002
 */
 
/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_AMP_ENVELOPE_H
#define	ANTHONY_AMP_ENVELOPE_H

#include	<vector>
#include	<utility>
#include	<cstdlib>
#include	"audio.h"

namespace AAF = AnthonyAudioFramework;

namespace	AnthonyAudioClasses	{

class	AmpEnvelope : public AAF::AudioAbstract
{
	public:
		AmpEnvelope() : AAF::AudioAbstract() { Init(); }
		AmpEnvelope( AAF::AudioAbstract* out, std::size_t port ) : AAF::AudioAbstract( out, port ) { Init(); }
		AmpEnvelope( AAF::AudioAbstract* out, std::size_t port, double dur, std::vector< std::pair<double,double> > stages );
		virtual ~AmpEnvelope();
		
		virtual std::size_t	PreviousCount()	{ return (std::size_t)1; }
		virtual std::size_t	GetSamples( AAF::AudioSample* buffer, std::size_t n );
		virtual void		Reset();
		
		// Will not accept durations greater than 27 hours * (44100/samplingrate)
		void		Duration( double duration )	{ _duration = duration; }	// duration is in seconds
		double	Duration()				{ return _duration; }
		
		// Each pair is a duration and an amplitude for a stage of the envelope.
		// All values are between 0.0 and 1.0. Durations are a fraction of the
		// total duration and should normally sum to 1.0.
		void		SetStages( std::vector< std::pair<double,double> > &stages );

	protected:
		virtual void	InitializeForPlay();

	private:
		std::size_t		_numStages;
		std::size_t		_currentStage;
		double*		_stageDurs;								// Size of these two arrays will be one more
		double*		_stageAmps;								// than _numStages due to the initial point.
		double		_duration;								// in seconds
		unsigned long	_samplecount;
		unsigned long*	_stageTimes;							// stage endpoints in samples
				
		void	Init();
		void	ResetState();
};

}	// namespace closure

#endif
