/*	SineWave.h

	A sine wave oscillator with high-precision frequency and 
	good signal-to-noise ratio since it calculates sin() for 
	each sample.
		
	Anthony Kozar
	October 8, 2002
	
	July 26, 2003:  modified to inherit from AbstractOscillator, which 
		removed the need for much of the existing code.
		
 */
 
/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_SINE_WAVE_H
#define	ANTHONY_SINE_WAVE_H

#include	<cstdlib>
#include	"AbstractOscillator.h"

namespace AAF = AnthonyAudioFramework;

namespace	AnthonyAudioClasses	{

const	double PI = 3.14159265358979;

class	SineWave : public AbstractOscillator //: public AAF::AudioAbstract
{
	public:
		SineWave();
		SineWave( AAF::AudioAbstract* out, std::size_t port );
		SineWave( AAF::AudioAbstract* out, std::size_t port, double frequency );
		SineWave( AAF::AudioAbstract* out, std::size_t port, double frequency, double amplitude );
		SineWave( AAF::AudioAbstract* out, std::size_t port, double frequency, double amplitude, double phase );
		//virtual ~SineWave();
	
/*		virtual std::size_t	PreviousCount()	{ return (size_t)0; }
		virtual std::size_t	GetSamples( AAF::AudioSample* buffer, std::size_t n );
		virtual void		Reset();
		
		void		Frequency( double freq )	{ _frequency = freq; }
		double	Frequency()				{ return _frequency; }
		void		Phase( double ph )		{ _phase = ph; }			// 0.0 <= ph < 1.0
		double	Phase()				{ return _phase; }
*/	
	protected:
//		virtual void InitializeForPlay();
		virtual double	GenerateSample( double freq, double amp, double pw );
		
	private:
/*		double	_frequency;
		double	_phase;
		double	_curPhase;
*/
};

}	// namespace closure

#endif
