/*	SquareWave.h

	A straight-forward square wave oscillator with variable pulse-width
	and precise frequency control.  No anti-aliasing however.
	(This is a "band-unlimited" waveform generator).
			
	Anthony Kozar
	October 9, 2002
	
	July 26, 2003: refactored most of SquareWave into a base class which it now inherits from.
	
 */
 
/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_SQUARE_WAVE_H
#define	ANTHONY_SQUARE_WAVE_H

#include	<cstdlib>
#include	"AbstractOscillator.h"
#include	"AbstractControlSource.h"

namespace AAF = AnthonyAudioFramework;

namespace	AnthonyAudioClasses	{

class	SquareWave : public AbstractOscillator //: public AAF::AudioAbstract
{
	public:
		SquareWave();
		SquareWave( AAF::AudioAbstract* out, std::size_t port );
		SquareWave( AAF::AudioAbstract* out, std::size_t port, double frequency );
		SquareWave( AAF::AudioAbstract* out, std::size_t port, double frequency, double amplitude );
		SquareWave( AAF::AudioAbstract* out, std::size_t port, double frequency, double amplitude, double pulseWidth );
		//virtual ~SquareWave();
	
/*		virtual std::size_t	PreviousCount()	{ return (size_t)0; }
		virtual std::size_t	GetSamples( AAF::AudioSample* buffer, std::size_t n );
		virtual void		Reset();
		
		void		Frequency( double freq )	{ _frequency = freq; }
		double	Frequency()				{ return _frequency; }
		void		Amplitude( double amp )		{ _amplitude = amp; }		// 0.0 <= _amp <= 1.0
		double	Amplitude()				{ return _amplitude; }
		void		PulseWidth( double pw )		{ _pulseWidth = pw; }		// 0.0 <= _pW <= 1.0
		double	PulseWidth()			{ return _pulseWidth; }
		void		Phase( double ph )		{ _phase = ph; }			// 0.0 <= ph < 1.0
		double	Phase()				{ return _phase; }
		
		void		FrequencyMod( AAF::AbstractControlSource* cs, double amount )	{ _freqModSrc = cs; _freqModAmt = amount; }
		void		AmplitudeMod( AAF::AbstractControlSource* cs, double amount )	{ _ampModSrc = cs; _ampModAmt = amount; }
		void		PulseWidthMod( AAF::AbstractControlSource* cs, double amount )	{ _pwModSrc = cs; _pwModAmt = amount; }
*/	
	protected:
//		virtual void InitializeForPlay();
		virtual double	GenerateSample( double freq, double amp, double pw );
		
	private:
/*		double	_frequency;
		double	_amplitude;
		double	_phase;
		double	_pulseWidth;
		double	_curPhase;
		
		AAF::AbstractControlSource*	_freqModSrc;
		double				_freqModAmt;
		AAF::AbstractControlSource*	_ampModSrc;
		double				_ampModAmt;
		AAF::AbstractControlSource*	_pwModSrc;
		double				_pwModAmt;
		
		void	InitMods();
		void	ReadMods( double& freq, double& amp, double& pw );
*/
};

}	// namespace closure

#endif
