/*	CSLFO.cpp    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include	"CSLFO.h"
#include	<cstdlib>
#include	<cmath>

using	namespace	std;


// constructors

CSLFO::CSLFO( unsigned long controlRate ) : AAF::AbstractControlSource(controlRate)
{
	_frequency = 1.0;
	_phase = _curPhase = 0.0;
	_freqModSrc = NULL;
	_freqModAmt = 0.0;
}
	
CSLFO::CSLFO( unsigned long controlRate, double frequency ) : AAF::AbstractControlSource(controlRate)
{
	_frequency = frequency;
	_phase = _curPhase = 0.0;
	_freqModSrc = NULL;
	_freqModAmt = 0.0;
}
	
CSLFO::CSLFO( unsigned long controlRate, double frequency, double phase ) : AAF::AbstractControlSource(controlRate)
{
	_frequency = frequency;
	_phase = _curPhase = phase;
	_freqModSrc = NULL;
	_freqModAmt = 0.0;
}
	

// overloaded virtual functions from AbstractControlSource interface

void	CSLFO::Update()
{
	AAF::AbstractControlSource::Update();							// call to superclass method
	UpdatePhase();
	Value(_curPhase);											// just output phase (unipolar sawwave)
	return;
}

		
void	CSLFO::UpdatePhase()
{
	double freq, phaseIncr;
	
	// calculate instantaneous frequency using mod source if any
	if	(_freqModSrc)	freq = _frequency + _freqModAmt * _freqModSrc->Read();
	else				freq = _frequency;
	
	// calculate the current oscillator phase, incrementing for continuity
	phaseIncr = freq / (double)ControlRate();
	_curPhase = fmod( _curPhase + phaseIncr, 1.0 );
	
	return;
}

		
void	CSLFO::Reset()
{
	AAF::AbstractControlSource::Reset();							// call to superclass method
	Value(_phase);
	_curPhase = _phase;
	
	return;
}		
