/*	PitchInitializer.h

	A class to set the frequency of an oscillator when
	a "Note On" is received.  Specify the relative course
	and fine tuning with respect to middle-C, give the 
	PitchInitializer a Midi note number, and it will set
	the frequency correctly for its attached oscillator.
	
	Anthony Kozar
	August 2, 2003
 */
 
/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_PITCH_INITIALIZER_H
#define	ANTHONY_PITCH_INITIALIZER_H
 
#include	"AbstractOscillator.h"
#include	"midi.h"

namespace	AAC = AnthonyAudioClasses;
namespace	AM = AnthonyMidi;
 
class	PitchInitializer
{
	public:
		PitchInitializer( AAC::AbstractOscillator* target, double course, double fine, double centsPerSemitone );
		
		AM::byte	MidiNote() const		{ return _noteNumber; }
		void		MidiNote( AM::byte nn )	{ _noteNumber = nn; }
		
		// call this function to set the Midi Note number and the oscillator's frequency
		void		SetPitch( AM::byte nn );
		
	private:
		AAC::AbstractOscillator*	_target;
		double 				_course;
		double 				_fine;
		double 				_perNote;
		AM::byte				_noteNumber;
};

#endif
