/*	au.cpp
	
	Based on software by and copyright Tim Kientzle, 1997.
	Changes and additions by Anthony M Kozar Jr.
*/

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
   Copyright 1997 Tim Kientzle.  All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. All advertising materials mentioning features or use of this software
   must display the following acknowledgement:
      This product includes software developed by Tim Kientzle
      and published in ``The Programmer's Guide to Sound.''
4. Neither the names of Tim Kientzle nor Addison-Wesley
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL TIM KIENTZLE OR ADDISON-WESLEY BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "au.h"
//#include "g711.h"

namespace	AnthonyAudioFramework	{

bool	IsAuFile( istream& file )
{
   file.seekg(0);  // Seek to beginning
   long magic = ReadIntMsb(file,4);
   return (magic == 0x2E736E64); // Should be `.snd'
}

/*
//	AuRead: constructors and destructors

AuRead::AuRead( istream& input = cin ) : AudioAbstract(), _stream(input)
{
	#ifndef NO_FEEDBACK									// added 111902, akozar.
	cerr << "File Format: Sun AU (also known as NeXT SND)\n";
	#endif

	_headerRead = false; 									// Haven't read header yet
	_decoder = 0;
}

AuRead::~AuRead()
{
	if (_decoder) delete _decoder;
}

//	AuRead: public member functions

size_t AuRead::GetSamples( AudioSample* buffer, size_t numSamples )
{
	return _decoder->GetSamples( buffer, numSamples );
}

size_t AuRead::ReadBytes( AudioByte* buffer, size_t length )
{
   if (length > _dataLength) { length = _dataLength; }
   _stream.read(reinterpret_cast<char *>(buffer),length);
   size_t lengthRead = _stream.gcount();
   _dataLength -= lengthRead;
   return lengthRead;
}

//	AuRead: protected member functions

void AuRead::MinMaxSamplingRate( long* min, long* max, long* preferred )
{
	ReadHeader();
	*min = *max = *preferred = _headerRate;
}

void AuRead::MinMaxChannels( int* min, int* max, int* preferred )
{
	ReadHeader();
	*min = *max = *preferred = _headerChannels;
}

//	AuRead: private member functions

void	AuRead::ReadHeader()
{
   if (_headerRead) return;
   _headerRead = true;

   char header[24];
   _stream.read(header,24);

   long magic = BytesToIntMsb(header+0,4);
   if (magic != 0x2E736E64) { // '.snd'
      cerr << "Input file is not an AU file.\n";
      exit(1);
   }

   long headerLength = BytesToIntMsb(header+4,4);
   _dataLength = BytesToIntMsb(header+8,4);
   int format = BytesToIntMsb(header+12,4);
   _headerRate = BytesToIntMsb(header+16,4);
   _headerChannels = BytesToIntMsb(header+20,4);
   SkipBytes(_stream,headerLength - 24); // Junk rest of header

   // Create an appropriate decompression object
   switch(format) {
   case 1: // ITU G.711 mu-Law
      _decoder = new DecompressG711MuLaw(*this);
      break;
   case 2: // 8-bit linear
      _decoder = new DecompressPcm8Unsigned(*this);
      break;
   case 3: // 16-bit linear
      _decoder = new DecompressPcm16MsbSigned(*this);
      break;
   default:
      cerr << "AU format " << format << " not supported.\n";
      exit(1);
   }

	#ifndef NO_FEEDBACK	// added 111902, akozar.
	cerr << "Sampling Rate: " << _headerRate;
	cerr << " Channels: " << _headerChannels;
	cerr << "\n";
	#endif
}
*/

/*	private functions for au.cpp    */

static void WriteBuffer( ostream& out, AudioSample* buffer, int length )
{
   AudioSample *sampleBuff = buffer;
   AudioByte *byteBuff = 
         reinterpret_cast<AudioByte *>(buffer);
   int i = length;
   while (i-->0) {
      int sample = *sampleBuff++;
      *byteBuff++ = sample >> 8;
      *byteBuff++ = sample;
   }
   out.write(reinterpret_cast<char *>(buffer),length*2);
}


/*	AuWrite: public member functions    */

void	AuWrite::Play( void (*serviceFunc)(void) )
{
   int samplingRate = SamplingRate();
   int channels = Channels();

   // give each audio object in the tree a chance to 
   // allocate memory before we start playing
   this->InitializeForPlay();
   
   // Write AU header
   _stream.write(".snd",4);									// Magic
   WriteIntMsb(_stream,28L,4);								// Length of header
   WriteIntMsb(_stream,0x7FFFFFFFL,4);							// length of data
   WriteIntMsb(_stream,3L,4);									// 16-bit linear
   WriteIntMsb(_stream,samplingRate,4);							// Sampling rate
   WriteIntMsb(_stream,channels,4);								// number of channels
   WriteIntMsb(_stream,0,4);									// padding

   // Read sound data and write it to the file
   long length;
   do {
      const int BuffSize = 10240;
      AudioSample buff[BuffSize];
      length = Previous(0)->GetSamples(buff,BuffSize);
      WriteBuffer(_stream,buff,length);
      if (serviceFunc)										// Modified 072402 by akozar to call
         serviceFunc(); 									// a service function like MacPlayer does.
   } while (length > 0);
}

}	// namespace closure

