/*	gentables.cpp    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include	"gentables.h"

const	double PI = 3.14159265358979;

using namespace std;
using namespace AnthonyAudioFramework;

void	SineTable( AudioSample* table, size_t tablesize )
{
	double	phase;
	AudioSample	value;
	//size_t	halfsize = tablesize / 2;
	
	for	( size_t index = 0; index < tablesize; index++ )	{
		//phase = (double)(index - halfsize) / (double)halfsize;
		phase = (double)index / (double)tablesize;
		value = (AudioSample)( AUDIOSAMPLE_MAX * sin(2*PI*phase) );
		table[index] = value;
	}
	
	return;
}

void	HyperbolicTanTable( AudioSample* table, size_t tablesize, double gain )
{
	double	phase;
	AudioSample	value;
	size_t	halfsize = tablesize / 2;
	
	for	( size_t index = 0; index < tablesize; index++ )	{
		phase = (double)(index - halfsize) / (double)halfsize;
		value = (AudioSample)( AUDIOSAMPLE_MAX * tanh(gain*phase) );
		table[index] = value;
	}
	
	return;
}

void	LinearClippingTable( AAF::AudioSample* table, std::size_t tablesize, double width, double offset )
{
	AudioSample	value;
	size_t	halfsize = tablesize / 2;
	size_t	windowwidth = (size_t)(tablesize * (1.0-width));
	size_t	windowoffset = (size_t)(halfsize * offset);
	size_t	lowerbound = (halfsize - (windowwidth/2)) + windowoffset;
	size_t	upperbound = (halfsize + (windowwidth/2)) + windowoffset;
	
	// constrain the bounds to within the table indices
	if	( lowerbound > tablesize )	lowerbound = 0;			// I use > because size_t is unsigned
	if	( upperbound > tablesize )	upperbound = tablesize;
	
	// calculate the table values
	for	( size_t index = 0; index < tablesize; index++ )	{
		if	( index < lowerbound )		value = AUDIOSAMPLE_MIN;
		else	if	( index > upperbound )	value = AUDIOSAMPLE_MAX;
		else	value = (AudioSample)( (double(index-lowerbound)/(double)windowwidth)*2.0 - 1.0 );
		table[index] = value;
	}
	
	return;
}
