/*	AM1Note.cpp    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include	"AM1Note.h"


// constructor

AM1Note::AM1Note()
{
	for	( int i = 0; i < NUM_OPERATORS; i++ )
		oscillators[i] = NULL;
	for	( int i = 0; i < NUM_MIXERS; i++ )
		mixers[i] = NULL;
	for	( int i = 0; i < NUM_WAVESHAPERS; i++ )
		shapers[i] = NULL;
	for	( int i = 0; i < NUM_OPERATORS; i++ )
		pinits[i] = NULL;
	
	//filter = NULL;
	amplifier = NULL;
	
	attVelocity = new CSConstant();
	relVelocity = new CSConstant();
	midiNoteNum = new CSConstant();
	
	controls = NULL;
	numControls = 0;
}

// public member functions

void	AM1Note::NoteOn( AM::byte midinote, AM::byte attvel )
{
	SetPitch(midinote);
	midiNoteNum->SetValue( (double)midinote / 127.0 );
	attVelocity->SetValue( (double)attvel / 127.0 );
	
	return;
}

void	AM1Note::NoteOff( AM::byte relvel )
{
	Release();
	relVelocity->SetValue( (double)relvel / 127.0 );

	return;
}

void	AM1Note::SetPitch( AM::byte midinote )
{
	for	( int i = 0; i < NUM_OPERATORS; i++ )
		if	( pinits[i] )	pinits[i]->SetPitch(midinote);
		
	return;
}

void	AM1Note::Release()
{
	amplifier->TriggerRelease();
	return;
}

void	AM1Note::UpdateControls()
{
	for	( int i = 0; i < numControls; i++ )
		if	(controls[i])	controls[i]->Update();
	
	return;
}

void	AM1Note::ResetControls()
{
	for	( int i = 0; i < numControls; i++ )
		if	(controls[i])	controls[i]->Reset();
	
	return;
}

