/*	AM1Note.h

	Class for coordinating events with synthesizer notes and
	for storing pointers to all of the objects involved.
	
	Anthony Kozar
	August 2, 2003
 */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_AM1_NOTE_H
#define	ANTHONY_AM1_NOTE_H

#include	"AudioAbstract.h"
#include	"AbstractOscillator.h"
#include	"AbstractMixer.h"
#include	"Waveshaper.h"
//#include	"SecondOrderFilter.h"
#include	"AmpEnvRelease.h"
#include	"midi.h"
#include	"PitchInitializer.h"
#include	"AbstractControlSource.h"
#include	"CSConstant.h"
#include	"AM1Instrument.h"

namespace	AAF = AnthonyAudioFramework;
namespace	AAC = AnthonyAudioClasses;
namespace	AM = AnthonyMidi;


class	AM1Note
{
	public:
		AM1Note();
		
		AAC::AbstractOscillator*	oscillators[NUM_OPERATORS];
		AAC::AbstractMixer*		mixers[NUM_MIXERS];
		AAC::Waveshaper*			shapers[NUM_WAVESHAPERS];
		//AAC::SecondOrderFilter*		filter;
		AAC::AmpEnvRelease*		amplifier;
		
		PitchInitializer*			pinits[NUM_OPERATORS];
		CSConstant*				attVelocity;
		CSConstant*				relVelocity;
		CSConstant*				midiNoteNum;
		
		AAF::AbstractControlSource**		controls;
		std::size_t					numControls;
		
		// call NoteOn() to initialize a note for playback
		void	NoteOn( AM::byte midinote, AM::byte attvel );
		
		// call NoteOff() to trigger end of note
		void	NoteOff( AM::byte relvel );
		
		// call to update or reset all control sources
		void	UpdateControls();
		void	ResetControls();

		// set pitch for all pinits and key follows
		void	SetPitch( AM::byte midinote );
		
		// trigger envelope releases
		void	Release();
};

#endif
