/*	AM1NoteFactory.cpp    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include	<climits>
#include	<cassert>
#include	<algorithm>

#include	"midifunc.h"
#include	"AM1NoteFactory.h"
#include	"AM1Instrument.h"

#include	"SineWave.h"
#include	"SquareWave.h"
#include	"SawWave.h"
#include	"SimpleMixer.h"
#include	"RingModulator.h"
#include	"ANDMixer.h"
#include	"ORMixer.h"
#include	"XORMixer.h"
#include	"AmplitudeModulator.h"
#include	"AmpEnvRelease.h"
#include	"Waveshaper.h"

#include	"CSMidiController.h"
#include	"CSMidiAftertouch.h"
#include	"CSLFO.h"
#include	"CSLFOSine.h"
#include	"CSLFOTriangle.h"
#include	"CSLFOSquare.h"
#include	"CSMult.h"


using	namespace	std;
using	namespace	AnthonyAudioFramework;
using	namespace	AnthonyAudioClasses;

#include	"algorithm_data.h"


// constructors

AM1NoteFactory::AM1NoteFactory()	{}

AM1NoteFactory::AM1NoteFactory( AM1Instrument* instrument )
{
	_instr = instrument;
	PrepareFactory();
}


// public member functions

void	AM1NoteFactory::SetInstrument( AM1Instrument* instrument )
{
	_instr = instrument;
	PrepareFactory();
}


AM1Note* AM1NoteFactory::BuildNote( AudioAbstract* output, size_t outPort )
{
	AM1Note*		theNote;
	AudioAbstract*	nextOut;
	size_t		nextPort;
	
	theNote = new AM1Note();
	
	// add the amplifier/amp envelope
	nextOut = theNote->amplifier = new AmpEnvRelease( output, outPort, _instr->envParms, _instr->envSustainStage, _instr->envReleaseStage );
	nextPort = 0;
	
	// add optional filter and waveshapers if necessary
	if	(_instr->filter.create)	{
		//nextOut = theNote->filter = new SecondOrderFilter( nextOut, nextPort, instr->filter.type, instr->filter.cutoff, instr->filter.resonance );
		//nextPort = 0;
	}
	if	(_instr->shapers[1].create)	{
		nextOut = theNote->shapers[1] = new Waveshaper( nextOut, nextPort, _instr->shapers[1].table );
		nextPort = 0;
	}
	if	(_instr->shapers[0].create)	{
		nextOut = theNote->shapers[0] = new Waveshaper( nextOut, nextPort, _instr->shapers[0].table );
		nextPort = 0;
	}
	
	BuildAlgorithm( nextOut, nextPort, *theNote );
	BuildControls(*theNote);
	
	return theNote;
}


// private member functions

void	AM1NoteFactory::PrepareFactory()
{
/*	int	numMods = _instr->modMatrix.size();
	bool*	examined = new bool(numMods);
	
	fill( examined, examined+numMods, false );
	for	( int i = 0; i < numMods; i++ )	{
		if	(!examined[i])	{
			
*/	
	return;
}

void	AM1NoteFactory::BuildAlgorithm( AudioAbstract* output, size_t outPort, AM1Note& note )
{
	bool			done = false;
	AudioAbstract*	objList[12];
	int			objCount = 0, oscCount = 0, mixCount = 0, vertexCount = 0;
	object_t		nextType;
	AudioAbstract*	nextOut;
	size_t		nextPort;
	size_t		mixerSize;
	
	double 
	freq = 200.0;
	
	while (!done)	{
		// get data from ModuleCreationRecords for the next module
		nextType = kAlgorithmData[_instr->algorithm][objCount].type;
		if	( kAlgorithmData[_instr->algorithm][objCount].output_to == kFirst )
			nextOut = output;
		else	nextOut = objList[ kAlgorithmData[_instr->algorithm][objCount].output_to ];
		if	( kAlgorithmData[_instr->algorithm][objCount].port == kFirst )
			nextPort = outPort;
		else	nextPort = kAlgorithmData[_instr->algorithm][objCount].port;
		mixerSize = kAlgorithmData[_instr->algorithm][objCount].numPorts;
		
		// create a module of the appropriate class and connect it
		switch (nextType)	{
			case	otNull:
				done = true;
				break;
				
			case	otOscillator:
				switch (_instr->oscillators[oscCount].type)	{
					case	sine:
						objList[objCount] = note.oscillators[oscCount] = new SineWave( nextOut, nextPort, freq, _instr->oscillators[oscCount].amplitude );
						break;
					case	saw:
						objList[objCount] = note.oscillators[oscCount] = new SawWave( nextOut, nextPort, freq, _instr->oscillators[oscCount].amplitude );
						break;
					case	sqr:
					default:
						objList[objCount] = note.oscillators[oscCount] = new SquareWave( nextOut, nextPort, freq, _instr->oscillators[oscCount].amplitude, _instr->oscillators[oscCount].pulsewidth );
						break;
				}	
				
				note.pinits[oscCount] = new PitchInitializer( note.oscillators[oscCount], 
											    _instr->oscillators[oscCount].courseTuning, 
											    _instr->oscillators[oscCount].fineTuning, 
											    _instr->oscillators[oscCount].centsPerSemitone );
				freq += 50.0;
				++oscCount;
				break;
				
			case	otAmpMod:
				objList[objCount] = note.mixers[mixCount] = new AmplitudeModulator( nextOut, nextPort );
				++mixCount;
				break;
				
			case	otMixer:
				switch (_instr->mix_types[vertexCount])	{
					case	mtRing:
						objList[objCount] = note.mixers[mixCount] = new RingModulator( nextOut, nextPort, mixerSize );
						break;
					case	mtAnd:
						objList[objCount] = note.mixers[mixCount] = new ANDMixer( nextOut, nextPort, mixerSize );
						break;
					case	mtOr:
						objList[objCount] = note.mixers[mixCount] = new ORMixer( nextOut, nextPort, mixerSize );
						break;
					case	mtXor:
						objList[objCount] = note.mixers[mixCount] = new XORMixer( nextOut, nextPort, mixerSize );
						break;
					case	mtAdd:
					default:
						objList[objCount] = note.mixers[mixCount] = new SimpleMixer( nextOut, nextPort, mixerSize );
						break;
				}		
				++mixCount; ++vertexCount;
				break;
				
			default:
				break;
		}
		
		++objCount;
		
	}
	
	return;
}


void	AM1NoteFactory::BuildControls( AM1Note& note )
{
	ControlPointers				cp;
	vector<AbstractControlSource*>	controlList;
	bool						madeSrc, madeDest;
	
	vector<AM1Instrument::ModulationRouting>::iterator		iter, iterEnd;
	
	// process each mod routing, one at a time
	iterEnd = _instr->modMatrix.end();
	for	( iter = _instr->modMatrix.begin(); iter != iterEnd; ++iter )	{
		madeSrc = false, madeDest = false;
		
		// get mod source
		AbstractControlSource*	source;
		modsources	srcType = iter->source;
		size_t	srcNum = iter->sourceNum;
		
		// create the source if it does not exist yet
		switch (srcType)	{
			case	msAttackVelocity:		source = (AbstractControlSource*)note.attVelocity; break;
			case	msReleaseVelocity:	source = (AbstractControlSource*)note.relVelocity; break;
			case	msNoteNum:			source = (AbstractControlSource*)note.midiNoteNum; break;
			default:				source = cp.GetPointer( srcType, srcNum ); break;
		}
		if	(!source)	{
			source = MakeControl( srcType, srcNum );
			if	(!source)	continue;					// if "bad" source, then skip to next mod
			else	madeSrc = true;
		}
		
		// get mod destination
		AbstractControlSource*	target;
		moddests	dest = iter->dest;
		modsources	destConv;
		dest_t	destType = iter->destType;
		size_t	destNum = iter->destNum;
		
		if	( DestIsControlSource(dest) )	{
			// create the destination if it does not exist yet
			destConv = moddests2modsources(dest);
			target = cp.GetPointer( destConv, destNum );
			if	(!target)	{
				target = MakeControl( destConv, destNum );
				if	(!target)	{					// if "bad" destination, then skip to next mod
					if	(madeSrc)	delete source;
					continue;
				}
				else	madeDest = true;
			}
			// link
			LinkModSrcToModDest( source, iter->amount, target, dest, destType );
		}
		else	LinkModSrcToAudioDest( source, iter->amount, note, dest, destType, destNum );
		
		// save pointers
		if	(madeSrc)	{								
			cp.SetPointer( srcType, srcNum, source );
			controlList.push_back(source);
		}
		if	(madeDest)	{
			cp.SetPointer( destConv, destNum, target );
			controlList.push_back(target);
		}
	}
	
	// convert controlList to an array of pointers
	AbstractControlSource**	alist = new AbstractControlSource*[controlList.size()];
	copy( controlList.begin(), controlList.end(), alist );
	
	// save pointer array to AM1Note
	note.controls = alist;
	note.numControls = controlList.size();
	
	return;
}

void	AM1NoteFactory::LinkModSrcToModDest( AbstractControlSource* src, double amt, AbstractControlSource* target, moddests dest, dest_t destType )
{
	switch (dest)	{
		case	mdLFO:
		case	mdGLFO:
			if	( destType == dtRate )	dynamic_cast<CSLFO*>(target)->FrequencyMod( src, amt );
			else	assert(true);
			break;
			
		case	mdEnv:
			break;
			
		case	mdFun:
			if		( destType == dtModA )	dynamic_cast<CSMult*>(target)->SourceAMod( src, amt );
			else	if	( destType == dtModB )	dynamic_cast<CSMult*>(target)->SourceBMod( src, amt );
			else	assert(true);
			break;
		
		default:
			assert(true);
			break;
	}
		
	return;
}

void	AM1NoteFactory::LinkModSrcToAudioDest( AbstractControlSource* src, double amt, AM1Note& note, moddests dest, dest_t destType, size_t destNum )
{
	switch (dest)	{
		case	mdOperator:
			if	(note.oscillators[destNum])	{
				if		( destType == dtFreq )	note.oscillators[destNum]->FrequencyMod( src, amt );
				else	if	( destType == dtAmp )	note.oscillators[destNum]->AmplitudeMod( src, amt );
				else	if	( destType == dtPWM )	note.oscillators[destNum]->PulseWidthMod( src, amt );
				else	assert(true);
			}
			break;
			
		case	mdShaper:
			break;
			
		case	mdFilter:
			break;
			
		case	mdAmp:
			if	(note.amplifier)	{
				//if		( destType == dtDecibels )	note.amplifier->AmplitudeMod( src, amt );
				//else	if	( destType == dtAttack )	note.amplifier->AttackMod( src, amt );
				//else	if	( destType == dtRelease )	note.amplifier->ReleaseMod( src, amt );
				//else	assert(true);
			}
			break;
		
		default:
			break;	
	}
	return;
}

AbstractControlSource*	AM1NoteFactory::MakeControl( modsources src, std::size_t num )
{
	AbstractControlSource*	cs = NULL;
	AM::byte			mchan = 0;		// midi channel
	
	switch (src)	{
		case	msAttackVelocity:
			//cs = new CSMidiAttVelocity( CONTROL_RATE );
			break;

		case	msReleaseVelocity:
			//cs = new CSMidiRelVelocity( CONTROL_RATE );
			break;

		case	msController:
			cs = new CSMidiController( CONTROL_RATE, mchan, num+1, ReadController );
			break;

		case	msAftertouch:
			cs = new CSMidiAftertouch( CONTROL_RATE, mchan, ReadAftertouch );
			break;

		case	msNoteNum:
			//cs = new CSMidiNoteNumber( CONTROL_RATE );
			break;

		case	msEnv:
			//cs = new CS4StageEnv( CONTROL_RATE );
			break;

		case	msLFO:
			cs = MakeLFO( num, false );
			break;

		case	msGLFO:
			cs = MakeLFO( num, true );
			break;

		case	msRnd:
			//cs = new CSRandom( CONTROL_RATE );
			break;

		case	msKey:
			//cs = new CSKeyFollow( CONTROL_RATE );
			break;

		case	msFun:
			cs = new CSMult( CONTROL_RATE );
			break;

		case	msClock:
			//cs = new CSMidiClock( CONTROL_RATE );
			break;
		
		default:
			cs = NULL;
			break;
	}
	
	return cs;
}

AbstractControlSource*	AM1NoteFactory::MakeLFO( std::size_t num, bool global )
{
	AbstractControlSource*		cs = NULL;
	AM1Instrument::LFO_Record*	rec;
	
	if	(global)	rec = &(_instr->globalLFOs[num]);
	else			rec = &(_instr->LFOs[num]);
	
	switch(rec->type)	{
		case	sine:
			cs = new CSLFOSine( CONTROL_RATE, rec->rate, rec->phase );
			break;
		case	tri:
			cs = new CSLFOTriangle( CONTROL_RATE, rec->rate, rec->phase );
			break;
		case	sqr:
			cs = new CSLFOSquare( CONTROL_RATE, rec->rate, rec->phase );
			break;
		case	saw:
			//cs = new CSLFOSaw( CONTROL_RATE, rec->rate, rec->phase );
			break;
		case	rnd:
			//cs = new CSLFORandom( CONTROL_RATE, rec->rate, rec->phase );
			break;
		default:
			cs = NULL;
			break;
	}

	return cs;
}

bool	AM1NoteFactory::DestIsControlSource( moddests type )
{
	switch (type)	{
		case	mdLFO:
		case	mdGLFO:
		case	mdEnv:
		case	mdFun:
			return true;
			break;
		default:
			return false;
			break;
	}
}

modsources	AM1NoteFactory::moddests2modsources( moddests type )
{
	switch (type)	{
		case	mdLFO:	return msLFO;  break;
		case	mdGLFO:	return msGLFO; break;
		case	mdEnv:	return msEnv;  break;
		case	mdFun:	return msFun;  break;
		default:		return (modsources)-1; break;
	}
}


// ControlPointers constructor & member functions

AM1NoteFactory::ControlPointers::ControlPointers()
{
	/*for	( int i = 0; i < NUM_AUX_ENVELOPES; i++)
		envelopes[i] = NULL;*/
	for	( int i = 0; i < NUM_LFOS; i++)
		LFOs[i] = NULL;
	for	( int i = 0; i < NUM_GLOBAL_LFOS; i++)
		GLFOs[i] = NULL;
	/*for	( int i = 0; i < NUM_KEY_FOLLOWS; i++)
		keyFollows[i] = NULL;*/
	for	( int i = 0; i < (NUM_AUX_ENVELOPES+NUM_LFOS+NUM_GLOBAL_LFOS); i++)
		depthFuns[i] = NULL;
}

AbstractControlSource*	AM1NoteFactory::ControlPointers::GetPointer( modsources src, size_t num ) const
{
	switch (src)	{
		/*case	msEnv:
			return envelopes[num};
			break;*/

		case	msLFO:
			return LFOs[num];
			break;

		case	msGLFO:
			return GLFOs[num];
			break;

		/*case	msKey:
			return keyFollows[num];
			break;*/

		case	msFun:
			return depthFuns[num];
			break;
		
		default:
			return NULL;
			break;
	}
}

void	AM1NoteFactory::ControlPointers::SetPointer( modsources src, std::size_t num, AbstractControlSource* cs )
{
	switch (src)	{
		/*case	msEnv:
			envelopes[num] = dynamic_cast<CS4StageEnv*>(cs);
			break;*/

		case	msLFO:
			LFOs[num] = dynamic_cast<CSLFO*>(cs);
			break;

		case	msGLFO:
			GLFOs[num] = dynamic_cast<CSLFO*>(cs);
			break;

		/*case	msKey:
			keyFollows[num] = dynamic_cast<CSKeyFollow*>(cs);
			break;*/

		case	msFun:
			depthFuns[num] = dynamic_cast<CSMult*>(cs);
			break;
		
		default:
			break;
	}
	
	return;
}

