/*	AM1NoteFactory.h

	A class to build AM1Note instances from the blueprint
	in an AM1Instrument object.  Inspired by the 
	"Abstract Factory" design pattern.
	
	Anthony Kozar
	July 26, 2003
 */
/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/



#ifndef	ANTHONY_AM1_NOTE_FACTORY_H
#define	ANTHONY_AM1_NOTE_FACTORY_H

#include	<cstdlib>

#include	"AM1Instrument.h"
#include	"AM1Note.h"
#include	"AudioAbstract.h"
#include	"AbstractControlSource.h"
#include	"CSLFO.h"
#include	"CSMult.h"


const	unsigned long	CONTROL_RATE = 42;

class	AM1NoteFactory
{
	public:
		AM1NoteFactory();
		AM1NoteFactory( AM1Instrument* instrument );
		
		void	SetInstrument( AM1Instrument* instrument );
	
		AM1Note*	BuildNote( AAF::AudioAbstract* output, std::size_t outPort );
					    
	private:
		AM1Instrument*	_instr;
		
		class ControlPointers
		{
			public:
			ControlPointers();
			
			AAF::AbstractControlSource*	GetPointer( modsources src, std::size_t num ) const;
			void	SetPointer( modsources src, std::size_t num, AAF::AbstractControlSource* cs );
			
			//CS4StageEnv*	envelopes[NUM_AUX_ENVELOPES];
			CSLFO*		LFOs[NUM_LFOS];
			CSLFO*		GLFOs[NUM_GLOBAL_LFOS];
			//CSKeyFollow*	keyFollows[NUM_KEY_FOLLOWS];
			CSMult*		depthFuns[NUM_AUX_ENVELOPES+NUM_LFOS+NUM_GLOBAL_LFOS];
		};
		
		void	PrepareFactory();
		void	BuildAlgorithm( AAF::AudioAbstract* output, std::size_t outPort, AM1Note& note );
		void	BuildControls( AM1Note& note );
		void	LinkModSrcToModDest( AAF::AbstractControlSource* src, double amt, AAF::AbstractControlSource* target, moddests dest, dest_t destType );
		void	LinkModSrcToAudioDest( AAF::AbstractControlSource* src, double amt, AM1Note& note, moddests dest, dest_t destType, std::size_t destNum );
		AAF::AbstractControlSource*	MakeControl( modsources src, std::size_t num );
		AAF::AbstractControlSource*	MakeLFO( std::size_t num, bool global );
		bool	DestIsControlSource( moddests type );
		modsources	moddests2modsources( moddests dest);
};

#endif
