/*	NoteName.cpp    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include	"NoteName.h"

#include	<sstream>
#include	<cstring>
#include	<cmath>

using	namespace	std;

const	double TWELFTH_ROOT_OF_TWO = 1.05946309436;
const	double CONCERT_PITCH = 440.00;

const	char	NOTENAME_STRINGS[21][3] = { "Cf", "C", "Cs", "Df", "D", "Ds", "Ef", "E", "Es", 
		"Ff", "F", "Fs", "Gf", "G", "Gs", "Af", "A", "As", "Bf", "B", "Bs" };
		
const	int	NAME_PITCHCLASS_MAP[21] = { 11, 0, 1, 1, 2, 3, 3, 4, 5, 4, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 0 };

const	char	PITCHCLASS_NAME_MAP[12][3] = { "C", "Cs", "D", "Ef", "E", "F", "Fs", "G", "Gs","A", "Bf", "B" };


NoteName::pitchclass NoteName::Pitchclass() const
{
	return _pclass;
}


int	NoteName::Octave() const
{
	return _octave;
}


NoteName& NoteName::Pitchclass( pitchclass value )
{
	_pclass = value;
	return *this;
}


NoteName& NoteName::Octave( int value )
{
	_octave = value;
	return *this;
}


NoteName::NoteName()
{
	_pclass = c;
	_octave = 0;
}


NoteName::~NoteName()
{

}


NoteName::NoteName( const NoteName& other )
{
	_pclass = other._pclass;
	_octave = other._octave;
}


NoteName::NoteName( string name )
{
	*this = String2NoteName(name);
}


NoteName::NoteName( pitchclass pc, int oct )
{
	_pclass = pc;
	_octave = oct;
}


NoteName& NoteName::operator=( const NoteName& other )
{
	_pclass = other._pclass;
	_octave = other._octave;

	return *this;
}


string NoteName::toString() const
{
	stringstream ss;
	
	ss	<< PITCHCLASS_NAME_MAP[(int)_pclass] << _octave;
	
	return ss.str();
}


double NoteName::toFrequency() const
{
	int	rel_concert_pitch = ((int)_pclass - 9);
	
	return ( CONCERT_PITCH * pow(2,(_octave - 4)) * pow(TWELFTH_ROOT_OF_TWO,rel_concert_pitch) ); 
}


double NoteName::toCsound() const
{
	return ( (double)_octave + (double)_pclass / 100.0 );

}


#ifdef	NOTENAME_USING_MIDI
AnthonyMidi::byte	NoteName::toMidiNote() const
{
	return AnthonyMidi::byte( (_octave+1)*12 + (int)_pclass );
}
#endif


NoteName NoteName::String2NoteName( std::string name )
{
	string	pcstr, octstr;
	NoteName	nn;
	
	if	( isdigit(name[1]) )	{						// if second char is a number
		pcstr = name.substr(0,1);						// then only first char is pitchclass
		octstr = name.substr(1);						// and the rest is the octave
	}
	else	{
		pcstr = name.substr(0,2);						// otherwise, 2-char. pitchclass
		octstr = name.substr(2);
	}
	
	for	( int i = 0; i < 21; i++ )	{
		if	( pcstr == NOTENAME_STRINGS[i] )	{
			nn.Pitchclass( (NoteName::pitchclass)NAME_PITCHCLASS_MAP[i] );
			break;
		}
	}
	nn.Octave( atoi(octstr.c_str()) );

	return nn;
}


// overloaded stream operators

ostream& operator<<( ostream& sout, const NoteName& nn )
{
	sout	<< nn.toString();
	return sout;
}

istream& operator>>( istream& sin, NoteName& nn )
{
	string	name;
	
	sin	>> name;
	nn = nn.String2NoteName(name);
	
	return sin;
}
