/*	NoteName.h

	Anthony Kozar
	October 10, 2002
 */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_NOTENAME_H
#define	ANTHONY_NOTENAME_H

#include	<iostream>
#include	<string>

#ifdef	NOTENAME_USING_MIDI
#include	"midi.h"
#endif

class	NoteName
{
	friend std::ostream& operator<<( std::ostream& sout, const NoteName& nn ); 
	friend std::istream& operator>>( std::istream& sin, NoteName& nn ); 
	
	public:
	
		enum	pitchclass	{ c, csharp, d, dsharp, e, f, fsharp, g, gsharp, a, asharp, b };
		
		NoteName();
		~NoteName();
		NoteName( const NoteName& other );
		NoteName( std::string name );
		NoteName( pitchclass pc, int oct );

		NoteName&	operator=( const NoteName& other );

		std::string	toString() const;
		double	toFrequency() const;
		double	toCsound() const;
		
#ifdef	NOTENAME_USING_MIDI
		AnthonyMidi::byte	toMidiNote() const;
#endif

		pitchclass	Pitchclass() const;
		int		Octave() const;

		NoteName&	Pitchclass( pitchclass value );
		NoteName&	Octave( int value );

	private:
		pitchclass	_pclass;
		int		_octave;
		
		NoteName	String2NoteName( std::string name );
};

#endif

