/*	algorithm_data.h    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

enum	object_t	{ otNull, otOscillator, otAmpMod, otMixer, otWaveshaper, otFilter, otAmplifier };


const	std::size_t			kFirst = INT_MAX;
const	std::size_t			kNumberOfAlgorithms = 16;
const	std::size_t			kRecordsPerAlgorithm = 8;

struct ModuleCreationRecord
{
	object_t			type;
	std::size_t			output_to;		// which (previous) object this one "plugs into" (kFirst for the first object which plugs into what comes after algorithm)
	std::size_t			port;			// the port on the output object to connect to
	std::size_t			numPorts;		// if type is otMixer, the number of ports to give it
	//AAF::AudioAbstract*	theObject;		// place to save the pointer
};


const	ModuleCreationRecord	kAlgorithmData [kNumberOfAlgorithms+1][kRecordsPerAlgorithm] = 
		{
			{ },			// leave first array blank
			
			// algorithm 1
			{ { otOscillator, kFirst, kFirst },
			  { otNull }
			},
			
			// algorithm 2
			{ { otMixer, kFirst, kFirst, 2 },
			  { otOscillator, 0, 0 },
			  { otOscillator, 0, 1 },
			  { otNull }
			},
			
			// algorithm 3
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otOscillator, 0, 1 },
			  { otNull }
			},
			
			// algorithm 4
			{ { otMixer, kFirst, kFirst, 3 },
			  { otOscillator, 0, 0 },
			  { otOscillator, 0, 1 },
			  { otOscillator, 0, 2 },
			  { otNull }
			},
			
			// algorithm 5
			{ { otMixer, kFirst, kFirst, 2 },
			  { otAmpMod, 0, 0 },
			  { otOscillator, 1, 0 },
			  { otOscillator, 1, 1 },
			  { otOscillator, 0, 1 },
			  { otNull }
			},
			
			// algorithm 6
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otMixer, 0, 1, 2 },
			  { otOscillator, 2, 0 },
			  { otOscillator, 2, 1 },
			  { otNull }
			},
			
			// algorithm 7
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otAmpMod, 0, 1 },
			  { otOscillator, 2, 0 },
			  { otOscillator, 2, 1 },
			  { otNull }
			},

			// algorithm 8
			{ { otMixer, kFirst, kFirst, 4 },
			  { otOscillator, 0, 0 },
			  { otOscillator, 0, 1 },
			  { otOscillator, 0, 2 },
			  { otOscillator, 0, 3 },
			  { otNull }
			},
			
			// algorithm 9
			{ { otMixer, kFirst, kFirst, 3 },
			  { otAmpMod, 0, 0 },
			  { otOscillator, 1, 0 },
			  { otOscillator, 1, 1 },
			  { otOscillator, 0, 1 },
			  { otOscillator, 0, 2 },
			  { otNull }
			},
			
			// algorithm 10
			{ { otMixer, kFirst, kFirst, 2 },
			  { otAmpMod, 0, 0 },
			  { otOscillator, 1, 0 },
			  { otMixer, 1, 1, 2 },
			  { otOscillator, 3, 0 },
			  { otOscillator, 3, 1 },
			  { otOscillator, 0, 1 },
			  { otNull }
			},
			
			// algorithm 11
			{ { otMixer, kFirst, kFirst, 2 },
			  { otAmpMod, 0, 0 },
			  { otOscillator, 1, 0 },
			  { otAmpMod, 1, 1 },
			  { otOscillator, 3, 0 },
			  { otOscillator, 3, 1 },
			  { otOscillator, 0, 1 },
			  { otNull }
			},

			// algorithm 12
			{ { otMixer, kFirst, kFirst, 2 },
			  { otAmpMod, 0, 0 },
			  { otOscillator, 1, 0 },
			  { otOscillator, 1, 1 },
			  { otAmpMod, 0, 1 },
			  { otOscillator, 4, 0 },
			  { otOscillator, 4, 1 },
			  { otNull }
			},
			
			// algorithm 13
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otMixer, 0, 1, 3 },
			  { otOscillator, 2, 0 },
			  { otOscillator, 2, 1 },
			  { otOscillator, 2, 2 },
			  { otNull }
			},
			
			// algorithm 14
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otMixer, 0, 1, 2 },
			  { otAmpMod, 2, 0 },
			  { otOscillator, 3, 0 },
			  { otOscillator, 3, 1 },
			  { otOscillator, 2, 1 },
			  { otNull }
			},
			
			// algorithm 15
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otAmpMod, 0, 1 },
			  { otOscillator, 2, 0 },
			  { otMixer, 2, 1, 2 },
			  { otOscillator, 4, 0 },
			  { otOscillator, 4, 1 },
			  { otNull }
			},
			
			// algorithm 16
			{ { otAmpMod, kFirst, kFirst },
			  { otOscillator, 0, 0 },
			  { otAmpMod, 0, 1 },
			  { otOscillator, 2, 0 },
			  { otAmpMod, 2, 1 },
			  { otOscillator, 4, 0 },
			  { otOscillator, 4, 1 },
			  { otNull }
			}
		};

