/*	macevent.cpp    */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include	"macevent.h"
#include	"macmain.h"
#include	<SIOUX.h>
#include	<cstdlib>

namespace	MacEvent	{

#define	HighWord(x)	(x>>16)
#define	LowWord(x)  (x & 0xFFFF)

// menu ID constants
#define	kAppleMenu		128
#define	kFileMenu		129
#define	kEditMenu		130
#define	kMidiMenu		131
#define	kOptionsMenu	132

// Apple menu item constants
#define	kAboutItem			1

// File menu item constants
#define	kOpenInstrumentItem	1
#define	kSaveAudioToDiskItem	2
#define	kQuitItem	4

// Midi menu item constants
#define	kSetMidiInputDeviceItem	1
//--------- dividing line --------- 2
#define	kAllNotesOffItem		3


void	EventLoop()
{
	while	(true)	CheckEvent();
}

void	CheckEvent()
{
	EventRecord	theEvent;
	bool		gotEvent;
	
	ProcessMidiPackets();
	
	gotEvent = WaitNextEvent(everyEvent, &theEvent, 1L, nil);
	
	if	( gotEvent and !SIOUXHandleOneEvent(&theEvent) ) {
		switch (theEvent.what)	{
			case	mouseDown:
				DoMouseDown(theEvent);
				break;
			
			case	mouseUp:
				break;
				
			case	keyDown:
			case	autoKey:
				DoKeyDown(theEvent);
				break;
				
			case updateEvt:
				DoUpdate(theEvent);
				break;
			
			default:
				break;
		}
	}
	
	return;
}

bool	DoMouseDown( EventRecord& evt )
{
	bool		evtHandled = false;
	short		windowPart;
	WindowPtr	whichWin;
	
	windowPart = FindWindow( evt.where, &whichWin );
	
	switch (windowPart)	{
		case	inMenuBar:
			long item = MenuSelect(evt.where);
			evtHandled = DoMenuItem( evt, item );
			break;
		
		case	inSysWindow:
			SystemClick( &evt, whichWin );
			evtHandled = true;
			break;
			
		case	inContent:											// one of our windows
			if	( whichWin == FrontWindow() )
				evtHandled = DoWindowClick( evt, whichWin );				// user event handler
			else	{
				SelectWindow(whichWin);
				evtHandled = true;
			}
			break;
			
		case	inDrag:
			DragWindow( whichWin, evt.where, &((**GetGrayRgn()).rgnBBox) );
			evtHandled = true;
			break;
			
		case	inGrow:
			/*const Rect	resizeBounds = { 64, 64, 65535, 65535 };
			long size = GrowWindow( whichWin, evt.where, &resizeBounds );
			SizeWindow( whichWin, HiWord(size), LoWord(size), true );			// change window size
			DoResizeWindow(whichWin);								// user resize/redraw routine
			evtHandled = true;
			*/
			break;
			
		case	inGoAway:
			if	( TrackGoAway( whichWin, evt.where ) )	{
				DoCloseItem(whichWin);								// user close routine
			}
			evtHandled = true;
			break;
			
		case	inZoomIn:
		case	inZoomOut:
			if	( TrackBox( whichWin, evt.where, windowPart ) )	{
				ZoomWindow( whichWin, windowPart, true );
			}
			evtHandled = true;
			break;
			
		default:
			break;
	}
	
	return evtHandled;
}


bool	DoKeyDown( EventRecord& evt )
{
	bool	evtHandled = false;
	
	// check if the key is a menu shortcut
	if	( evt.modifiers & cmdKey )	{
		long item = MenuKey( char(evt.message & 0x000000FF) );
		if	( HiWord(item) )	{
			evtHandled = DoMenuItem( evt, item );
		}
		else	return false;
	}
	
	// handle other keyboard events here
	
	return evtHandled;
}


bool	DoMenuItem( EventRecord& evt, long item )
{
	bool	evtHandled = false;
	long	menuID, mItemID;
	
	menuID = HighWord(item);
	mItemID = LowWord(item);
	
	switch (menuID)	{
		case	kAppleMenu:
			if	( mItemID == kAboutItem )	DoAbout();
			else	{
				Str255 	name;
				MenuHandle	menu;
				
				menu = GetMenuHandle(menuID);
				GetMenuItemText( menu, mItemID, name );
				OpenDeskAcc(name);
			}
			HiliteMenu(0);
			evtHandled = true;
			break;
		
		case	kFileMenu:
			switch (mItemID)	{
				case	kQuitItem:			DoQuit(); break;
				case	kOpenInstrumentItem:	DoOpenInstrumentItem(); break;
				case	kSaveAudioToDiskItem:	DoSaveAudioToDiskItem(); break;
				default:				break;
			}
			HiliteMenu(0);	
			break;
			
		case	kEditMenu:
			break;
		
		case	kMidiMenu:
			switch (mItemID)	{
				case	kSetMidiInputDeviceItem:	DoSetMidiInputDeviceItem(); break;
				case	kAllNotesOffItem:			DoAllNotesOff(); break;
				default:					break;
			}
			HiliteMenu(0);
			break;
			
		default:
			HiliteMenu(0);
			break;
	}
	
	return evtHandled;
}



/*			case mouseDown:
			winPart = FindWindow(theEvent.where, &whichWin);
			switch (winPart) {
				case inGoAway:
				if (TrackGoAway(whichWin, theEvent.where))
					DoQuit();
				break;
				
				case inContent:
				pt = theEvent.where;
				SetPort(whichWin);
				GlobalToLocal(&pt);
				if (TestOMSDeviceMenu(gInputMenu, pt)) {
					if (ClickOMSDeviceMenu(gInputMenu)) {
						InputMenuSideEffects();
					}
				} else if (TestOMSDeviceMenu(gOutputMenu, pt)) {
					if (ClickOMSDeviceMenu(gOutputMenu)) {
						OutputMenuSideEffects();
					}
				}
				break;
				
			}
			break;
			
			case app4Evt:
			if (theEvent.message & 0x01000000) {	// suspend/resume
				if (theEvent.message & 0x00000001) {
					OMSResume(MySignature);
					// add whatever else to resume your application 
				} else {
					OMSSuspend(MySignature);
					// add whatever else to suspend your application
				}
			}
			break;
		}
	} else {
		// null event
		
	}
}
*/			


}	// namespace closure
