/*	midi.h

	Basic Midi data types, storage, and functions.
	
	Anthony Kozar
	March 23, 2003
	
 */

/*
	The AM1 Synthesizer for Macintosh, version 0.50 beta
	Copyright (C) 2003 The University of Toledo.
	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:

	1.    Redistributions of source code must retain the above copyright notice,
	      this list of conditions and the following disclaimer. 
	2.    Redistributions in binary form must reproduce the above copyright notice,
	      this list of conditions and the following disclaimer in the documentation
	      and/or other materials provided with the distribution. 
	3.    The names of the authors and of the University of Toledo may not be used
	      to endorse or promote products derived from this software without specific
	      prior written permission. 

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND THE UNIVERSITY OF TOLEDO "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR THE UNIVERSITY OF TOLEDO BE LIABLE
	FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
	DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
	SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
	CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
	TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef	ANTHONY_MIDI
#define	ANTHONY_MIDI

namespace	AnthonyMidi	{

typedef	unsigned char	byte;
typedef	unsigned short	int14;

const	int14		kPitchBendCenter		= 0x2000;
const	byte		kMiddleC			= 60;

const	byte		maskMidiStatusMessage	= 0xF0;
const byte		maskMidiChannel		= 0x0F;
const byte		kMidiNoteOff		= 0x80;
const byte		kMidiNoteOn			= 0x90;
const byte		kMidiPolyAftertouch	= 0xA0;
const byte		kMidiController		= 0xB0;
const byte		kMidiProgramChange	= 0xC0;
const byte		kMidiChannelPressure	= 0xD0;
const byte		kMidiPitchBend		= 0xE0;

enum	MidiControllers	{	
			kMidiBankSelect		= 0,
			kMidiModWheel		= 1,
			kMidiBreathController	= 2,
			kMidiFootController	= 4,
			kMidiPortamentoTime	= 5,
			kMidiDataEntry		= 6,
			kMidiChannelVolume	= 7,
			kMidiBalance		= 8,
			kMidiPan			= 10,
			kMidiExpression		= 11,
			kMidiEffect1		= 12,
			kMidiEffect2		= 13,
			kMidiGeneralPurpose1	= 16,
			kMidiGeneralPurpose2	= 17,
			kMidiGeneralPurpose3	= 18,
			kMidiGeneralPurpose4	= 19,
			kMidiBankSelect32		= 32,
			kMidiSustainPedal		= 64,
			kMidiPortamento		= 65,
			kMidiSustenuto		= 66,
			kMidiSoftPedal		= 67,
			kMidiAllSoundOff		= 120,
			kMidiResetAllControllers= 121,
			kMidiAllNotesOff		= 123
};


class	ChannelStatus
{
	public:
		ChannelStatus();
		ChannelStatus( byte which_channel );
		
		void	Reset();
		
		byte	channel;
		bool	activeNotes[128];
		byte	activeProgram;
		int14	pitchbend;
		byte	monoPressure;
		byte	controller[128];
		//byte	polyPressure[128];
		
};

int14	Make14bitInteger( byte low, byte high );

	
}	// namespace closure

#endif